@@ -23,10 +23,11 @@
 #include "FeaturedModel.h"
 #include "CachedNetworkAccessManager.h"
 #include "DiscoverDeclarativePlugin.h"
+#include "DiscoverBackendsFactory.h"
 
 // Qt includes
 #include <QAction>
-#include <QDebug>
+#include "discover_debug.h"
 #include <QDesktopServices>
 #include <QtQml/QQmlEngine>
 #include <QtQml/QQmlContext>
@@ -38,6 +39,7 @@
 #include <QSortFilterProxyModel>
 #include <QTimer>
 #include <QSessionManager>
+#include <QClipboard>
 
 // KDE includes
 #include <KAboutApplicationDialog>
@@ -65,6 +67,7 @@
 #include <unistd.h>
 #include <resources/StoredResultsStream.h>
 #include <utils.h>
+#include <QMimeDatabase>
 
 class OurSortFilterProxyModel : public QSortFilterProxyModel, public QQmlParserStatus
 {
@@ -108,7 +111,6 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
     qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
 
-    qmlRegisterSingletonType(QUrl(QStringLiteral("qrc:/qml/DiscoverSystemPalette.qml")), "org.kde.discover.app", 1, 0, "DiscoverSystemPalette");
     qmlRegisterType<QQuickView>();
     qmlRegisterType<QActionGroup>();
     qmlRegisterType<QAction>();
@@ -128,6 +130,12 @@ DiscoverObject::DiscoverObject(CompactMode mode)
 
     connect(m_engine, &QQmlApplicationEngine::objectCreated, this, &DiscoverObject::integrateObject);
     m_engine->load(QUrl(QStringLiteral("qrc:/qml/DiscoverWindow.qml")));
+
+    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, this, [this](){
+        const auto objs = m_engine->rootObjects();
+        for(auto o: objs)
+            delete o;
+    });
 }
 
 DiscoverObject::~DiscoverObject()
@@ -158,12 +166,12 @@ void DiscoverObject::openMode(const QString& _mode)
 {
     QObject* obj = rootObject();
     if (!obj) {
-        qWarning() << "could not get the main object";
+        qCWarning(DISCOVER_LOG) << "could not get the main object";
         return;
     }
 
     if(!modes().contains(_mode, Qt::CaseInsensitive))
-        qWarning() << "unknown mode" << _mode << modes();
+        qCWarning(DISCOVER_LOG) << "unknown mode" << _mode << modes();
 
     QString mode = _mode;
     mode[0] = mode[0].toUpper();
@@ -171,7 +179,7 @@ void DiscoverObject::openMode(const QString& _mode)
     const QByteArray propertyName = "top"+mode.toLatin1()+"Comp";
     const QVariant modeComp = obj->property(propertyName.constData());
     if (!modeComp.isValid())
-        qWarning() << "couldn't open mode" << _mode;
+        qCWarning(DISCOVER_LOG) << "couldn't open mode" << _mode;
     else
         obj->setProperty("currentTopLevel", modeComp);
 }
@@ -197,7 +205,7 @@ void DiscoverObject::openCategory(const QString& category)
         , this);
 
     if (ResourcesModel::global()->backends().isEmpty()) {
-        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+        connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, action, &OneTimeAction::trigger);
     } else {
         action->trigger();
     }
@@ -207,25 +215,33 @@ void DiscoverObject::openLocalPackage(const QUrl& localfile)
 {
     if (!QFile::exists(localfile.toLocalFile())) {
 //         showPassiveNotification(i18n("Trying to open unexisting file '%1'", localfile.toString()));
-        qWarning() << "Trying to open unexisting file" << localfile;
+        qCWarning(DISCOVER_LOG) << "Trying to open unexisting file" << localfile;
         return;
     }
     setRootObjectProperty("defaultStartup", false);
     auto action = new OneTimeAction(
         [this, localfile]() {
             auto res = ResourcesModel::global()->resourceForFile(localfile);
-            qDebug() << "all initialized..." << res;
+            qCDebug(DISCOVER_LOG) << "all initialized..." << res;
             if (res) {
                 emit openApplicationInternal(res);
             } else {
-                setRootObjectProperty("defaultStartup", true);
-                showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                QMimeDatabase db;
+                auto mime = db.mimeTypeForUrl(localfile);
+                auto fIsFlatpakBackend = [](AbstractResourcesBackend* backend) { return backend->metaObject()->className() == QByteArray("FlatpakBackend"); };
+                if (mime.name().startsWith(QLatin1String("application/vnd.flatpak")) && !kContains(ResourcesModel::global()->backends(), fIsFlatpakBackend)) {
+                    openApplication(QUrl(QLatin1String("appstream://org.kde.discover.flatpak")));
+                    showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.", localfile.toDisplayString()));
+                } else {
+                    setRootObjectProperty("defaultStartup", true);
+                    showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                }
             }
         }
         , this);
 
     if (ResourcesModel::global()->backends().isEmpty()) {
-        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+        connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, action, &OneTimeAction::trigger);
     } else {
         action->trigger();
     }
@@ -246,14 +262,14 @@ void DiscoverObject::openApplication(const QUrl& url)
                     emit openApplicationInternal(res.first());
                 } else {
                     setRootObjectProperty("defaultStartup", true);
-                    showPassiveNotification(i18n("Couldn't open %1", url.toDisplayString()));
+                    Q_EMIT openErrorPage(i18n("Couldn't open %1", url.toDisplayString()));
                 }
             });
         }
         , this);
 
     if (ResourcesModel::global()->backends().isEmpty()) {
-        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+        connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, action, &OneTimeAction::trigger);
     } else {
         action->trigger();
     }
@@ -262,7 +278,7 @@ void DiscoverObject::openApplication(const QUrl& url)
 void DiscoverObject::integrateObject(QObject* object)
 {
     if (!object) {
-        qWarning() << "Errors when loading the GUI";
+        qCWarning(DISCOVER_LOG) << "Errors when loading the GUI";
         QTimer::singleShot(0, QCoreApplication::instance(), [](){
             QCoreApplication::instance()->exit(1);
         });
@@ -282,6 +298,7 @@ void DiscoverObject::integrateObject(QObject* object)
     object->installEventFilter(this);
     connect(object, &QObject::destroyed, qGuiApp, &QCoreApplication::quit);
 
+    object->setParent(m_engine);
     connect(qGuiApp, &QGuiApplication::commitDataRequest, this, [this](QSessionManager &sessionManager) {
         if (ResourcesModel::global()->isBusy()) {
             Q_EMIT preventedClose();
@@ -297,7 +314,7 @@ bool DiscoverObject::eventFilter(QObject * object, QEvent * event)
 
     if (event->type() == QEvent::Close) {
         if (ResourcesModel::global()->isBusy()) {
-            qWarning() << "not closing because there's still pending tasks";
+            qCWarning(DISCOVER_LOG) << "not closing because there's still pending tasks";
             Q_EMIT preventedClose();
             return true;
         }
@@ -306,7 +323,7 @@ bool DiscoverObject::eventFilter(QObject * object, QEvent * event)
         window.writeEntry("geometry", rootObject()->geometry());
         window.writeEntry<int>("visibility", rootObject()->visibility());
 //     } else if (event->type() == QEvent::ShortcutOverride) {
-//         qWarning() << "Action conflict" << event;
+//         qCWarning(DISCOVER_LOG) << "Action conflict" << event;
     }
     return false;
 }
@@ -381,7 +398,7 @@ public:
         m_testObject = component->create(engine->rootContext());
 
         if (!m_testObject) {
-            qWarning() << "error loading test" << url << m_testObject << component->errors();
+            qCWarning(DISCOVER_LOG) << "error loading test" << url << m_testObject << component->errors();
             Q_ASSERT(false);
         }
 
@@ -392,7 +409,7 @@ public:
     void processWarnings(const QList<QQmlError> &warnings) {
         foreach(const QQmlError &warning, warnings) {
             if (warning.url().path().endsWith(QLatin1String("DiscoverTest.qml"))) {
-                qWarning() << "Test failed!" << warnings;
+                qCWarning(DISCOVER_LOG) << "Test failed!" << warnings;
                 qGuiApp->exit(1);
             }
         }
@@ -406,9 +423,9 @@ public:
         }));
 
         if (m_warnings.isEmpty())
-            qDebug() << "cool no warnings!";
+            qCDebug(DISCOVER_LOG) << "cool no warnings!";
         else
-            qDebug() << "test finished succesfully despite" << m_warnings;
+            qCDebug(DISCOVER_LOG) << "test finished succesfully despite" << m_warnings;
         qGuiApp->exit(m_warnings.count());
     }
 
@@ -431,7 +448,7 @@ void DiscoverObject::setRootObjectProperty(const char* name, const QVariant& val
 {
     auto ro = rootObject();
     if (!ro) {
-        qWarning() << "please check your installation";
+        qCWarning(DISCOVER_LOG) << "please check your installation";
         return;
     }
 
@@ -445,4 +462,10 @@ void DiscoverObject::showPassiveNotification(const QString& msg)
     });
 }
 
+void DiscoverObject::copyTextToClipboard(const QString& text)
+{
+    qGuiApp->clipboard()->setText(text);
+}
+
+
 #include "DiscoverObject.moc"
