@@ -22,7 +22,7 @@
 #include "CategoryModel.h"
 #include "Category.h"
 #include "CategoriesReader.h"
-#include <QDebug>
+#include "libdiscover_debug.h"
 #include <QCollator>
 #include <utils.h>
 #include <resources/ResourcesModel.h>
@@ -49,6 +49,9 @@ void CategoryModel::populateCategories()
     QVector<Category*> ret;
     CategoriesReader cr;
     Q_FOREACH (const auto backend, backends) {
+        if (!backend->isValid())
+            continue;
+
         const QVector<Category*> cats = cr.loadCategoriesFile(backend);
 
         if(ret.isEmpty()) {
@@ -71,7 +74,10 @@ QVariantList CategoryModel::rootCategoriesVL() const
 
 void CategoryModel::blacklistPlugin(const QString &name)
 {
-    Category::blacklistPluginsInVector({name}, m_rootCategories);
+    const bool ret = Category::blacklistPluginsInVector({name}, m_rootCategories);
+    if (ret) {
+        Q_EMIT rootCategoriesChanged();
+    }
 }
 
 static Category* recFindCategory(Category* root, const QString& name)
