@@ -23,7 +23,7 @@
 // Qt includes
 #include <QFont>
 #include <QTimer>
-#include <QDebug>
+#include "libdiscover_debug.h"
 
 // KDE includes
 #include <KFormat>
@@ -50,7 +50,11 @@ UpdateModel::UpdateModel(QObject *parent)
     connect(m_updateSizeTimer, &QTimer::timeout, this, &UpdateModel::updateSizeChanged);
 }
 
-UpdateModel::~UpdateModel() = default;
+UpdateModel::~UpdateModel()
+{
+    qDeleteAll(m_updateItems);
+    m_updateItems.clear();
+}
 
 QHash<int,QByteArray> UpdateModel::roleNames() const
 {
@@ -87,7 +91,7 @@ void UpdateModel::resourceHasProgressed(AbstractResource* res, qreal progress)
     item->setProgress(progress);
 
     const QModelIndex idx = indexFromItem(item);
-    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole });
+    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole, SectionResourceProgressRole });
 }
 
 void UpdateModel::activityChanged()
@@ -97,7 +101,7 @@ void UpdateModel::activityChanged()
             m_updates->prepare();
             setResources(m_updates->toUpdate());
 
-            for(auto item : m_updateItems) {
+            for(auto item : qAsConst(m_updateItems)) {
                 item->setProgress(0);
             }
         } else
@@ -130,8 +134,17 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
         return item->progress();
     case ChangelogRole:
         return item->changelog();
-    case SectionRole:
-        return item->section();
+    case SectionRole: {
+        static const QString appUpdatesSection = i18nc("@item:inlistbox", "Application Updates");
+        static const QString systemUpdateSection = i18nc("@item:inlistbox", "System Updates");
+        switch(item->section()) {
+            case UpdateItem::ApplicationSection: return appUpdatesSection;
+            case UpdateItem::SystemSection: return systemUpdateSection;
+        }
+        return {};
+    } 
+    case SectionResourceProgressRole:
+        return (100-item->progress()) + (101 * item->section());
     default:
         break;
     }
@@ -181,13 +194,13 @@ bool UpdateModel::setData(const QModelIndex &idx, const QVariant &value, int rol
     return false;
 }
 
-void UpdateModel::fetchChangelog(int row)
+void UpdateModel::fetchUpdateDetails(int row)
 {
     UpdateItem *item = itemFromIndex(index(row, 0));
     Q_ASSERT(item);
     if (!item) return;
 
-    item->app()->fetchChangelog();
+    item->app()->fetchUpdateDetails();
 }
 
 void UpdateModel::integrateChangelog(const QString &changelog)
@@ -216,8 +229,6 @@ void UpdateModel::setResources(const QList<AbstractResource*>& resources)
     qDeleteAll(m_updateItems);
     m_updateItems.clear();
 
-    const QString appUpdatesSection = i18nc("@item:inlistbox", "Application Updates");
-    const QString systemUpdateSection = i18nc("@item:inlistbox", "System Updates");
     QVector<UpdateItem*> appItems, systemItems;
     foreach(AbstractResource* res, resources) {
         connect(res, &AbstractResource::changelogFetched, this, &UpdateModel::integrateChangelog, Qt::UniqueConnection);
@@ -225,10 +236,10 @@ void UpdateModel::setResources(const QList<AbstractResource*>& resources)
         UpdateItem *updateItem = new UpdateItem(res);
 
         if(!res->isTechnical()) {
-            updateItem->setSection(appUpdatesSection);
+            updateItem->setSection(UpdateItem::ApplicationSection);
             appItems += updateItem;
         } else {
-            updateItem->setSection(systemUpdateSection);
+            updateItem->setSection(UpdateItem::SystemSection);
             systemItems += updateItem;
         }
     }
@@ -314,9 +325,9 @@ void UpdateModel::resourceDataChanged(AbstractResource* res, const QVector<QByte
 
     const auto index = indexFromItem(item);
     if (properties.contains("state"))
-        dataChanged(index, index, {SizeRole, VersionRole});
+        Q_EMIT dataChanged(index, index, {SizeRole, VersionRole});
     else if (properties.contains("size")) {
-        dataChanged(index, index, {SizeRole});
+        Q_EMIT dataChanged(index, index, {SizeRole});
         m_updateSizeTimer->start();
     }
 }
