@@ -57,10 +57,12 @@
 #include <QNetworkAccessManager>
 
 #include <glib.h>
+#include <QRegularExpression>
+#include "FlatpakSourcesBackend.h"
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
-static QString idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref)
+static FlatpakResource::Id idForInstalledRef(FlatpakInstallation *installation, FlatpakInstalledRef *ref)
 {
     const FlatpakResource::ResourceType appType = flatpak_ref_get_kind(FLATPAK_REF(ref)) == FLATPAK_REF_KIND_APP ? FlatpakResource::DesktopApp : FlatpakResource::Runtime;
     const QString name = QLatin1String(flatpak_ref_get_name(FLATPAK_REF(ref)));
@@ -69,8 +71,7 @@ static QString idForInstalledRef(FlatpakInstallation *installation, FlatpakInsta
     const QString arch = QString::fromUtf8(flatpak_ref_get_arch(FLATPAK_REF(ref)));
     const QString branch = QString::fromUtf8(flatpak_ref_get_branch(FLATPAK_REF(ref)));
 
-    return QStringLiteral("%1/%2/%3/%4/%5/%6").arg(FlatpakResource::installationPath(installation), QLatin1String("flatpak"), QString::fromUtf8(flatpak_installed_ref_get_origin(ref)),
-                                                   FlatpakResource::typeAsString(appType), appId, branch);
+    return { installation, QString::fromUtf8(flatpak_installed_ref_get_origin(ref)), appType, appId, branch, arch };
 }
 
 FlatpakBackend::FlatpakBackend(QObject* parent)
@@ -78,10 +79,10 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(AppStreamIntegration::global()->reviews())
     , m_refreshAppstreamMetadataJobs(0)
+    , m_cancellable(g_cancellable_new())
     , m_threadPool(new QThreadPool(this))
 {
     g_autoptr(GError) error = nullptr;
-    m_cancellable = g_cancellable_new();
 
     connect(m_updater, &StandardBackendUpdater::updatesCountChanged, this, &FlatpakBackend::updatesCountChanged);
 
@@ -100,6 +101,8 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
 
 FlatpakBackend::~FlatpakBackend()
 {
+    g_cancellable_cancel(m_cancellable);
+    m_threadPool.waitForDone(200);
     m_threadPool.clear();
     for(auto inst : m_installations)
         g_object_unref(inst);
@@ -157,9 +160,12 @@ public:
                         resource->setResourceFile(originalUrl);
                         Q_EMIT jobFinished(true, resource);
                     } else {
-                        qWarning() << "couldn't download" << originalUrl << "into" << fileUrl << replyPut->errorString();
+                        qWarning() << "couldn't create resource from" << fileUrl.toLocalFile();
                         Q_EMIT jobFinished(false, nullptr);
                     }
+                } else {
+                    qWarning() << "couldn't save" << originalUrl << replyPut->errorString();
+                    Q_EMIT jobFinished(false, nullptr);
                 }
             });
         });
@@ -224,16 +230,18 @@ FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) co
         return runtime;
     }
 
-    const QString runtimeId = QStringLiteral("runtime/") + runtimeInfo.at(0) + QLatin1Char('/') + runtimeInfo.at(2);
-
     for(auto it = m_resources.constBegin(), itEnd = m_resources.constEnd(); it!=itEnd; ++it) {
-        if (it.key().endsWith(runtimeId)) {
+        const auto id = it.key();
+        if (id.type == FlatpakResource::Runtime && id.id == runtimeInfo.at(0) && id.branch == runtimeInfo.at(2)) {
             runtime = *it;
             break;
         }
     }
 
     // TODO if runtime wasn't found, create a new one from available info
+    if (!runtime) {
+        qWarning() << "could not find runtime" << runtimeInfo << resource;
+    }
 
     return runtime;
 }
@@ -364,7 +372,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
     const QString refurl = settings.value(QStringLiteral("Flatpak Ref/Url")).toString();
 
-    g_autoptr(GError) error = NULL;
+    g_autoptr(GError) error = nullptr;
     g_autoptr(FlatpakRemoteRef) remoteRef = nullptr;
     {
         QFile f(url.toLocalFile());
@@ -378,7 +386,11 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
 
         remoteRef = flatpak_installation_install_ref_file (preferredInstallation(), bytes, m_cancellable, &error);
         if (!remoteRef) {
-            qWarning() << "Failed to install ref file:" << error->message;
+            qWarning() << "Failed to create install ref file:" << error->message;
+            const auto resources = resourcesByAppstreamName(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+            if (!resources.isEmpty()) {
+                return qobject_cast<FlatpakResource*>(resources.constFirst());
+            }
             return nullptr;
         }
     }
@@ -412,7 +424,6 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
         auto installation = preferredInstallation();
         // We need to fetch metadata to find information about required runtime
         auto fw = new QFutureWatcher<QByteArray>(this);
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
         connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, installation, resource, fw, runtimeUrl]() {
             const auto metadata = fw->result();
             // Even when we failed to fetch information about runtime we still want to show the application
@@ -438,6 +449,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
             }
             fw->deleteLater();
         });
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
     } else {
         addResource(resource);
     }
@@ -511,11 +523,6 @@ void FlatpakBackend::addResource(FlatpakResource *resource)
     m_resources.insert(resource->uniqueId(), resource);
 }
 
-bool FlatpakBackend::compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref) const
-{
-    return resource->uniqueId() == idForInstalledRef(flatpakInstallation, ref);
-}
-
 class FlatpakSource
 {
 public:
@@ -601,19 +608,7 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
     }
 
     auto fw = new QFutureWatcher<QList<AppStream::Component>>(this);
-    fw->setFuture(QtConcurrent::run(&m_threadPool, [appDirFileName]() -> QList<AppStream::Component> {
-        AppStream::Metadata metadata;
-        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
-        AppStream::Metadata::MetadataError error = metadata.parseFile(appDirFileName, AppStream::Metadata::FormatKindXml);
-        if (error != AppStream::Metadata::MetadataErrorNoError) {
-            qWarning() << "Failed to parse appstream metadata: " << error;
-            return {};
-        }
-
-        return metadata.components();
-    }));
     const auto sourceName = source.name();
-    acquireFetching(true);
     connect(fw, &QFutureWatcher<QList<AppStream::Component>>::finished, this, [this, fw, flatpakInstallation, appstreamIconsPath, sourceName]() {
         const auto components = fw->result();
         foreach (const AppStream::Component& appstreamComponent, components) {
@@ -630,6 +625,18 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
         acquireFetching(false);
         fw->deleteLater();
     });
+    acquireFetching(true);
+    fw->setFuture(QtConcurrent::run(&m_threadPool, [appDirFileName]() -> QList<AppStream::Component> {
+        AppStream::Metadata metadata;
+        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+        AppStream::Metadata::MetadataError error = metadata.parseFile(appDirFileName, AppStream::Metadata::FormatKindXml);
+        if (error != AppStream::Metadata::MetadataErrorNoError) {
+            qWarning() << "Failed to parse appstream metadata: " << error;
+            return {};
+        }
+
+        return metadata.components();
+    }));
 }
 
 void FlatpakBackend::loadInstalledApps()
@@ -729,20 +736,19 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
 {
     auto fw = new QFutureWatcher<GPtrArray *>(this);
-    fw->setFuture(QtConcurrent::run(&m_threadPool, [installation]() -> GPtrArray * {
-        g_autoptr(GCancellable) cancellable = g_cancellable_new();
-        g_autoptr(GError) localError = nullptr;
-        GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, cancellable, &localError);
-        if (!refs) {
-            qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
-        }
-        return refs;
-    }));
     connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
         auto refs = fw->result();
         onFetchUpdatesFinished(installation, refs);
         fw->deleteLater();
     });
+    fw->setFuture(QtConcurrent::run(&m_threadPool, [installation, this]() -> GPtrArray * {
+        g_autoptr(GError) localError = nullptr;
+        GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, m_cancellable, &localError);
+        if (!refs) {
+            qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
+        }
+        return refs;
+    }));
 }
 
 void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstallation, GPtrArray *updates)
@@ -896,13 +902,13 @@ bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation,
         return updateAppMetadata(resource, path);
     } else {
         auto fw = new QFutureWatcher<QByteArray>(this);
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
         connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, flatpakInstallation, resource, fw]() {
             const auto metadata = fw->result();
             if (!metadata.isEmpty())
                 onFetchMetadataFinished(flatpakInstallation, resource, metadata);
             fw->deleteLater();
         });
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
 
         // Return false to indicate we cannot continue (right now used only in updateAppSize())
         return false;
@@ -932,21 +938,14 @@ bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QString
 
 bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QByteArray &data)
 {
-    // Save the content to temporary file
-    QTemporaryFile tempFile;
-    tempFile.setAutoRemove(false);
-    if (!tempFile.open()) {
-        qWarning() << "Failed to get metadata file";
+    //We just find the runtime with a regex, QSettings only can read from disk (and so does KConfig)
+    const QRegularExpression rx(QStringLiteral("runtime=(.*)"));
+    const auto match = rx.match(QString::fromUtf8(data));
+    if (!match.isValid()) {
         return false;
     }
 
-    tempFile.write(data);
-    tempFile.close();
-
-    updateAppMetadata(resource, tempFile.fileName());
-
-    tempFile.remove();
-
+    resource->setRuntime(match.captured(1));
     return true;
 }
 
@@ -1013,7 +1012,6 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
         }
 
         auto futureWatcher = new QFutureWatcher<FlatpakRunnables::SizeInformation>(this);
-        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
         connect(futureWatcher, &QFutureWatcher<FlatpakRunnables::SizeInformation>::finished, this, [this, resource, futureWatcher]() {
             auto value = futureWatcher->result();
             if (value.valid) {
@@ -1024,6 +1022,7 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
             }
             futureWatcher->deleteLater();
         });
+        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
     }
 
     return true;
@@ -1038,11 +1037,10 @@ void FlatpakBackend::onFetchSizeFinished(FlatpakResource *resource, guint64 down
 
     if (runtime && !runtime->isInstalled()) {
         resource->setDownloadSize(runtime->downloadSize() + downloadSize);
-        resource->setInstalledSize(installedSize);
     } else {
         resource->setDownloadSize(downloadSize);
-        resource->setInstalledSize(installedSize);
     }
+    resource->setInstalledSize(installedSize);
 }
 
 void FlatpakBackend::updateAppState(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
@@ -1077,10 +1075,10 @@ int FlatpakBackend::updatesCount() const
     return m_updater->updatesCount();
 }
 
-bool FlatpakBackend::flatpakResourceLessThan(AbstractResource* l, AbstractResource* r)
+bool FlatpakBackend::flatpakResourceLessThan(AbstractResource* l, AbstractResource* r) const
 {
     return (l->isInstalled() != r->isInstalled()) ? l->isInstalled()
-         : (l->origin() != r->origin()) ? l->origin() < r->origin()
+         : (l->origin() != r->origin()) ? m_sources->originIndex(l->origin()) < m_sources->originIndex(r->origin())
          : l < r;
 }
 
@@ -1105,22 +1103,45 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
     } else if (!filter.resourceUrl.isEmpty() || !filter.extends.isEmpty())
         return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
 
-    QVector<AbstractResource*> ret;
-    foreach(AbstractResource* r, m_resources) {
-        if (r->isTechnical() && filter.state != AbstractResource::Upgradeable) {
-            continue;
-        }
+    auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
+    auto f = [this, stream, filter] () {
+        QVector<AbstractResource*> ret;
+        foreach(AbstractResource* r, m_resources) {
+            if (r->isTechnical() && filter.state != AbstractResource::Upgradeable) {
+                continue;
+            }
 
-        if (r->state() < filter.state)
-            continue;
+            if (r->state() < filter.state)
+                continue;
 
-        if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
-            ret += r;
+            if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
+                ret += r;
+            }
         }
+        auto f = [this](AbstractResource* l, AbstractResource* r) { return flatpakResourceLessThan(l,r); };
+        std::sort(ret.begin(), ret.end(), f);
+        if (!ret.isEmpty())
+            Q_EMIT stream->resourcesFound(ret);
+        stream->finish();
+    };
+    if (isFetching()) {
+        connect(this, &FlatpakBackend::initialized, stream, f);
+    } else {
+        QTimer::singleShot(0, this, f);
     }
-    auto f = [this](AbstractResource* l, AbstractResource* r) { return flatpakResourceLessThan(l,r); };
-    std::sort(ret.begin(), ret.end(), f);
-    return new ResultsStream(QStringLiteral("FlatpakStream"), ret);
+    return stream;
+}
+
+QVector<AbstractResource *> FlatpakBackend::resourcesByAppstreamName(const QString& name) const
+{
+    QVector<AbstractResource*> resources;
+    foreach(FlatpakResource* res, m_resources) {
+        if (QString::compare(res->appstreamId(), name, Qt::CaseInsensitive)==0 || QString::compare(res->appstreamId(), name + QLatin1String(".desktop"), Qt::CaseInsensitive)==0)
+            resources << res;
+    }
+    auto f = [this](AbstractResource* l, AbstractResource* r) { return flatpakResourceLessThan(l, r); };
+    std::sort(resources.begin(), resources.end(), f);
+    return resources;
 }
 
 ResultsStream * FlatpakBackend::findResourceByPackageName(const QUrl &url)
@@ -1131,27 +1152,17 @@ ResultsStream * FlatpakBackend::findResourceByPackageName(const QUrl &url)
         else {
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
             auto f = [this, stream, url] () {
-                QVector<AbstractResource*> resources;
-                foreach(FlatpakResource* res, m_resources) {
-                    if (QString::compare(res->appstreamId(), url.host(), Qt::CaseInsensitive)==0)
-                        resources << res;
-                }
-                auto f = [this](AbstractResource* l, AbstractResource* r) { return flatpakResourceLessThan(l,r); };
-                std::sort(resources.begin(), resources.end(), f);
-
-                QTimer::singleShot(0, stream, [resources, stream] () {
-                    if (!resources.isEmpty())
-                        Q_EMIT stream->resourcesFound(resources);
-                    stream->finish();
-                });
+                const auto resources = resourcesByAppstreamName(url.host());
+                if (!resources.isEmpty())
+                    Q_EMIT stream->resourcesFound(resources);
+                stream->finish();
             };
 
             if (isFetching()) {
                 connect(this, &FlatpakBackend::initialized, stream, f);
             } else {
-                f();
+                QTimer::singleShot(0, this, f);
             }
-
             return stream;
         }
     }
