@@ -50,14 +50,14 @@ public:
     AbstractBackendUpdater * backendUpdater() const override;
     AbstractReviewsBackend * reviewsBackend() const override;
     ResultsStream * search(const AbstractResourcesBackend::Filters & search) override;
-    ResultsStream * findResourceByPackageName(const QUrl &search) override;
+    ResultsStream * findResourceByPackageName(const QUrl &search);
     QList<FlatpakResource*> resources() const { return m_resources.values(); }
     bool isValid() const override;
 
     Transaction* installApplication(AbstractResource* app) override;
     Transaction* installApplication(AbstractResource* app, const AddonList& addons) override;
     Transaction* removeApplication(AbstractResource* app) override;
-    bool isFetching() const override { return m_fetching; }
+    bool isFetching() const override { return m_isFetching>0; }
     AbstractResource * resourceForFile(const QUrl & ) override;
     void checkForUpdates() override;
     QString displayName() const override;
@@ -73,7 +73,7 @@ Q_SIGNALS: //for tests
     void initialized();
 
 private:
-    bool flatpakResourceLessThan(AbstractResource* l, AbstractResource* r);
+    bool flatpakResourceLessThan(AbstractResource* l, AbstractResource* r) const;
     void announceRatingsReady();
     FlatpakInstallation * preferredInstallation() const { return m_installations.constFirst(); }
     void integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
@@ -85,8 +85,6 @@ private:
     FlatpakResource * addAppFromFlatpakBundle(const QUrl &url);
     FlatpakResource * addAppFromFlatpakRef(const QUrl &url);
     void addResource(FlatpakResource *resource);
-    bool compareAppFlatpakRef(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, FlatpakInstalledRef *ref) const;
-    void finishInitialization();
     void loadAppsFromAppstreamData();
     bool loadAppsFromAppstreamData(FlatpakInstallation *flatpakInstallation);
     void loadInstalledApps();
@@ -104,13 +102,14 @@ private:
     bool updateAppSizeFromRemote(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource);
     void updateAppState(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource);
 
-    void setFetching(bool fetching);
+    QVector<AbstractResource*> resourcesByAppstreamName(const QString &name) const;
+    void acquireFetching(bool f);
 
-    QHash<QString, FlatpakResource*> m_resources;
+    QHash<FlatpakResource::Id, FlatpakResource*> m_resources;
     StandardBackendUpdater  *m_updater;
     FlatpakSourcesBackend *m_sources = nullptr;
     QSharedPointer<OdrsReviewsBackend> m_reviews;
-    bool m_fetching;
+    uint m_isFetching = 0;
     uint m_refreshAppstreamMetadataJobs;
 
     GCancellable *m_cancellable;
