@@ -34,8 +34,8 @@
 #include <knewstuffcore_version.h>
 #include <KNSCore/Engine>
 #include <KNSCore/QuestionManager>
+#include <KConfig>
 #include <KConfigGroup>
-#include <KDesktopFile>
 #include <KLocalizedString>
 
 // DiscoverCommon includes
@@ -71,7 +71,10 @@ class KNSBackendFactory : public AbstractResourcesBackendFactory {
                     dirIt.next();
 
                     auto bk = new KNSBackend(parent, QStringLiteral("plasma"), dirIt.filePath());
-                    ret += bk;
+                    if (bk->isValid())
+                        ret += bk;
+                    else
+                        delete bk;
                 }
             }
             return ret;
@@ -125,13 +128,25 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
         this->setFetching(false);
         qWarning() << "kns error" << objectName() << error;
         if (!invalidFile)
-            passiveMessage(i18n("%1: %2", name(), error));
+            Q_EMIT passiveMessage(i18n("%1: %2", name(), error));
     });
     connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
     connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged, Qt::QueuedConnection);
     connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, this, &KNSBackend::statusChanged);
     connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
 
+    // This ensures we have something to track when checking after the initialization timeout
+    connect(this, &KNSBackend::initialized, this, [this](){ m_initialized = true; });
+    // If we have not initialized in 60 seconds, consider this KNS backend invalid
+    QTimer::singleShot(60000, this, [this](){
+        if(!m_initialized) {
+            markInvalid(i18n("Backend %1 took too long to initialize", m_displayName));
+            m_responsePending = false;
+            Q_EMIT searchFinished();
+            Q_EMIT availableForQueries();
+        }
+    });
+
     const QVector<QPair<FilterType, QString>> filters = { {CategoryFilter, fileName } };
     const QSet<QString> backendName = { name() };
     m_displayName = group.readEntry("Name", QString());
@@ -158,7 +173,10 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
     m_rootCategories = { addonsCategory };
 }
 
-KNSBackend::~KNSBackend() = default;
+KNSBackend::~KNSBackend()
+{
+    qDeleteAll(m_rootCategories);
+}
 
 void KNSBackend::markInvalid(const QString &message)
 {
@@ -223,7 +241,7 @@ void KNSBackend::receivedEntries(const KNSCore::EntryInternal::List& entries)
         Q_EMIT receivedResources(resources);
     }
 
-    if(resources.isEmpty() || m_stopSearching) {
+    if(resources.isEmpty()) {
         Q_EMIT searchFinished();
         Q_EMIT availableForQueries();
         setFetching(false);
@@ -260,13 +278,22 @@ public:
         connect(manager, &KNSCore::Engine::signalEntryChanged, this, &KNSTransaction::anEntryChanged);
         TransactionModel::global()->addTransaction(this);
 
-        QTimer::singleShot(0, this, [this, res]() {
-            auto engine = res->knsBackend()->engine();
-            if (this->role() == InstallRole)
-                engine->install(res->entry());
-            else if(this->role() == RemoveRole)
+        std::function<void()> actionFunction;
+        auto engine = res->knsBackend()->engine();
+        if(role == RemoveRole)
+            actionFunction = [res, engine]() {
                 engine->uninstall(res->entry());
-        });
+            };
+        else if (res->linkIds().isEmpty())
+            actionFunction = [res, engine]() {
+                engine->install(res->entry());
+            };
+        else
+            actionFunction = [res, engine]() {
+                for(auto i : res->linkIds())
+                    engine->install(res->entry(), i);
+            };
+        QTimer::singleShot(0, res, actionFunction);
     }
 
     void anEntryChanged(const KNSCore::EntryInternal& entry) {
@@ -345,7 +372,7 @@ ResultsStream* KNSBackend::search(const AbstractResourcesBackend::Filters& filte
                 const auto ret = kFilter<QVector<AbstractResource*>>(m_resourcesByName, filterFunction);
 
                 if (!ret.isEmpty())
-                    stream->resourcesFound(ret);
+                    Q_EMIT stream->resourcesFound(ret);
             }
             stream->finish();
         };
@@ -379,12 +406,6 @@ void KNSBackend::searchStream(ResultsStream* stream, const QString &searchText)
         m_onePage = false;
         m_responsePending = true;
 
-        QSharedPointer<int> count(new int(0));
-        connect(this, &KNSBackend::receivedResources, stream, [this, count](const QVector<AbstractResource*>& resources){
-            *count += resources.count();
-            if (*count>2000)
-                m_stopSearching = true;
-        });
         connect(this, &KNSBackend::receivedResources, stream, &ResultsStream::resourcesFound);
         connect(this, &KNSBackend::searchFinished, stream, &ResultsStream::finish);
         connect(this, &KNSBackend::startingSearch, stream, &ResultsStream::finish);
@@ -406,7 +427,7 @@ ResultsStream * KNSBackend::findResourceByPackageName(const QUrl& search)
 
     const auto pathParts = search.path().split(QLatin1Char('/'), QString::SkipEmptyParts);
     if (pathParts.size() != 2) {
-        passiveMessage(i18n("Wrong KNewStuff URI: %1", search.toString()));
+        Q_EMIT passiveMessage(i18n("Wrong KNewStuff URI: %1", search.toString()));
         return voidStream();
     }
     const auto providerid = pathParts.at(0);
@@ -420,7 +441,7 @@ ResultsStream * KNSBackend::findResourceByPackageName(const QUrl& search)
         connect(m_engine, &KNSCore::Engine::signalError, stream, &ResultsStream::finish);
         connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, stream, [this, stream, entryid, providerid](const KNSCore::EntryInternal &entry) {
             if (entry.uniqueId() == entryid && providerid == QUrl(entry.providerId()).host()) {
-                stream->resourcesFound({resourceForEntry(entry)});
+                Q_EMIT stream->resourcesFound({resourceForEntry(entry)});
             } else
                 qWarning() << "found invalid" << entryid << entry.uniqueId() << providerid << QUrl(entry.providerId()).host();
             m_responsePending = false;
