@@ -84,7 +84,6 @@ private:
     void searchStream(ResultsStream* stream, const QString &searchText);
     
     bool m_onePage = false;
-    bool m_stopSearching = false;
     bool m_responsePending = false;
     bool m_fetching;
     bool m_isValid;
@@ -98,6 +97,7 @@ private:
     QStringList m_categories;
     QVector<Category*> m_rootCategories;
     QString m_displayName;
+    bool m_initialized = false;
 };
 
 #endif // KNSBACKEND_H
