@@ -85,7 +85,7 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     t->start();
 
     m_delayedDetailsFetch.setSingleShot(true);
-    m_delayedDetailsFetch.setInterval(0);
+    m_delayedDetailsFetch.setInterval(100);
     connect(&m_delayedDetailsFetch, &QTimer::timeout, this, &PackageKitBackend::performDetailsFetch);
 
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::updatesChanged, this, &PackageKitBackend::fetchUpdates);
@@ -99,6 +99,8 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     setWhenAvailable(PackageKit::Daemon::getTimeSinceAction(PackageKit::Transaction::RoleRefreshCache), [this](uint timeSince) {
         if (timeSince > 3600)
             checkForUpdates();
+        else
+            fetchUpdates();
     }, this);
 }
 
@@ -236,7 +238,6 @@ void PackageKitBackend::resolvePackages(const QStringList &packageNames)
 
     TransactionSet* merge = new TransactionSet({tArch, tNotArch});
     connect(merge, &TransactionSet::allFinished, this, &PackageKitBackend::getPackagesFinished);
-    fetchUpdates();
 }
 
 void PackageKitBackend::fetchUpdates()
@@ -337,11 +338,11 @@ void PackageKitBackend::packageDetails(const PackageKit::Details& details)
 
 QSet<AbstractResource*> PackageKitBackend::resourcesByPackageName(const QString& name) const
 {
-    return resourcesByPackageNames<QSet<AbstractResource*>, QVector<QString>>({name});
+    return resourcesByPackageNames<QSet<AbstractResource*>>({name});
 }
 
-template <typename T, typename Q>
-T PackageKitBackend::resourcesByPackageNames(const Q &pkgnames) const
+template <typename T>
+T PackageKitBackend::resourcesByPackageNames(const QStringList &pkgnames) const
 {
     T ret;
     ret.reserve(pkgnames.size());
@@ -393,7 +394,7 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
         if (!ids.isEmpty()) {
             const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(ids);
             QTimer::singleShot(0, this, [stream, resources] () {
-                stream->resourcesFound(resources);
+                Q_EMIT stream->resourcesFound(resources);
             });
         }
 
@@ -407,8 +408,8 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
             if (status == PackageKit::Transaction::Exit::ExitSuccess) {
                 const auto packageId = stream->property("packageId");
                 if (!packageId.isNull()) {
-                    const auto res = resourcesByPackageNames<QVector<AbstractResource*>, QVector<QString>>({PackageKit::Daemon::packageName(packageId.toString())});
-                    stream->resourcesFound(kFilter<QVector<AbstractResource*>>(res, [ids](AbstractResource* res){ return !ids.contains(res->appstreamId()); }));
+                    const auto res = resourcesByPackageNames<QVector<AbstractResource*>>({PackageKit::Daemon::packageName(packageId.toString())});
+                    Q_EMIT stream->resourcesFound(kFilter<QVector<AbstractResource*>>(res, [ids](AbstractResource* res){ return !ids.contains(res->appstreamId()); }));
                 }
             }
             stream->finish();
@@ -420,7 +421,9 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
 ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
 {
     AbstractResource* pkg = nullptr;
-    if (url.scheme() == QLatin1String("appstream")) {
+    if (url.host().isEmpty())
+        Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
+    else if (url.scheme() == QLatin1String("appstream")) {
         static const QMap<QString, QString> deprecatedAppstreamIds = {
             { QStringLiteral("org.kde.krita.desktop"), QStringLiteral("krita.desktop") },
             { QStringLiteral("org.kde.digikam.desktop"), QStringLiteral("digikam.desktop") },
@@ -433,11 +436,13 @@ ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
         
         const auto host = url.host();
         if (host.isEmpty())
-            passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
+            Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
-            const auto deprecatedHost = deprecatedAppstreamIds.value(url.host()); //try this as fallback
+            const auto deprecatedHost = deprecatedAppstreamIds.value(host); //try this as fallback
             for (auto it = m_packages.packages.constBegin(), itEnd = m_packages.packages.constEnd(); it != itEnd; ++it) {
-                if (it.key().compare(host, Qt::CaseInsensitive) == 0 || it.key().compare(deprecatedHost, Qt::CaseInsensitive) == 0) {
+                if    (it.key().compare(host, Qt::CaseInsensitive) == 0
+                    || it.key().compare(deprecatedHost, Qt::CaseInsensitive) == 0
+                    || (host.endsWith(QLatin1String(".desktop")) && host.compare(it.key()+QLatin1String(".desktop"), Qt::CaseInsensitive) == 0)) {
                     pkg = it.value();
                     break;
                 }
@@ -510,6 +515,10 @@ Transaction* PackageKitBackend::removeApplication(AbstractResource* app)
 
 QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
 {
+    if (isFetching() || !m_packagesToAdd.isEmpty()) {
+        return {};
+    }
+
     QSet<AbstractResource*> ret;
     ret.reserve(m_updatesPackageId.size());
     Q_FOREACH (const QString& pkgid, m_updatesPackageId) {
@@ -539,10 +548,7 @@ void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, c
 void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
 {
     if (!m_updatesPackageId.isEmpty()) {
-        PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(m_updatesPackageId.toList());
-        connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
-        connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-        connect(transaction, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesDetailsFinished);
+        fetchDetails(m_updatesPackageId);
     }
 
     m_updater->setProgressing(false);
@@ -551,13 +557,6 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
     emit updatesCountChanged();
 }
 
-void PackageKitBackend::getUpdatesDetailsFinished(PackageKit::Transaction::Exit exit, uint)
-{
-    if (exit != PackageKit::Transaction::ExitSuccess) {
-        qWarning() << "Couldn't figure out the updates on PackageKit backend" << exit;
-    }
-}
-
 bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource* res) const
 {
     return !upgradeablePackageId(res).isEmpty();
@@ -573,7 +572,7 @@ QString PackageKitBackend::upgradeablePackageId(const PackageKitResource* res) c
     return QString();
 }
 
-void PackageKitBackend::fetchDetails(const QString& pkgid)
+void PackageKitBackend::fetchDetails(const QSet<QString>& pkgid)
 {
     if (!m_delayedDetailsFetch.isActive()) {
         m_delayedDetailsFetch.start();
@@ -590,6 +589,7 @@ void PackageKitBackend::performDetailsFetch()
     PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
     connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+    m_packageNamesToFetchDetails.clear();
 }
 
 void PackageKitBackend::checkDaemonRunning()
