@@ -28,6 +28,8 @@
 #include <QJsonArray>
 #include <QDebug>
 
+const QStringList PackageKitResource::m_objects({ QStringLiteral("qrc:/qml/DependenciesButton.qml") });
+
 PackageKitResource::PackageKitResource(QString packageName, QString summary, PackageKitBackend* parent)
     : AbstractResource(parent)
     , m_summary(std::move(summary))
@@ -132,7 +134,7 @@ QString PackageKitResource::section()
     return QString();
 }
 
-AbstractResource::State PackageKitResource::state() const
+AbstractResource::State PackageKitResource::state()
 {
     if (backend()->isPackageNameUpgradeable(this))
         return Upgradeable;
@@ -197,12 +199,21 @@ void PackageKitResource::setDetails(const PackageKit::Details & details)
         emit stateChanged();
 
         if (!backend()->isFetching())
-            backend()->resourcesChanged(this, {"size", "homepage", "license"});
+            Q_EMIT backend()->resourcesChanged(this, {"size", "homepage", "license"});
     }
 }
 
 void PackageKitResource::fetchChangelog()
 {
+}
+
+void PackageKitResource::fetchUpdateDetails()
+{
+    const auto pkgid = availablePackageId();
+    if (pkgid.isEmpty()) {
+        connect(this, &PackageKitResource::stateChanged, this, &PackageKitResource::fetchUpdateDetails);
+        return;
+    }
     PackageKit::Transaction* t = PackageKit::Daemon::getUpdateDetail(availablePackageId());
     connect(t, &PackageKit::Transaction::updateDetail, this, &PackageKitResource::updateDetail);
     connect(t, &PackageKit::Transaction::errorCode, this, [this](PackageKit::Transaction::Error err, const QString & error) { qWarning() << "error fetching updates:" << err << error; emit changelogFetched(QString()); });
@@ -214,11 +225,15 @@ static void addIfNotEmpty(const QString& title, const QString& content, QString&
         where += QStringLiteral("<p><b>") + title + QStringLiteral("</b>&nbsp;") + QString(content).replace(QStringLiteral("\n"), QStringLiteral("<br />")) + QStringLiteral("</p>");
 }
 
-QString PackageKitResource::joinPackages(const QStringList& pkgids, const QString &_sep)
+QString PackageKitResource::joinPackages(const QStringList& pkgids, const QString &_sep, const QString &shadowPackage)
 {
     QStringList ret;
     foreach(const QString& pkgid, pkgids) {
-        ret += i18nc("package-name (version)", "%1 (%2)", PackageKit::Daemon::packageName(pkgid), PackageKit::Daemon::packageVersion(pkgid));
+        const auto pkgname = PackageKit::Daemon::packageName(pkgid);
+        if (pkgname == shadowPackage)
+            ret += PackageKit::Daemon::packageVersion(pkgid);
+        else
+            ret += i18nc("package-name (version)", "%1 (%2)", pkgname, PackageKit::Daemon::packageVersion(pkgid));
     }
     const QString sep = _sep.isEmpty() ? i18nc("comma separating package names", ", ") : _sep;
     return ret.join(sep);
@@ -232,21 +247,23 @@ static QStringList urlToLinks(const QStringList& urls)
     return ret;
 }
 
-void PackageKitResource::updateDetail(const QString& /*packageID*/, const QStringList& updates, const QStringList& obsoletes, const QStringList& vendorUrls,
+void PackageKitResource::updateDetail(const QString& packageID, const QStringList& updates, const QStringList& obsoletes, const QStringList& vendorUrls,
                                       const QStringList& /*bugzillaUrls*/, const QStringList& /*cveUrls*/, PackageKit::Transaction::Restart restart, const QString& updateText,
                                       const QString& /*changelog*/, PackageKit::Transaction::UpdateState state, const QDateTime& /*issued*/, const QDateTime& /*updated*/)
 {
+    const auto name = PackageKit::Daemon::packageName(packageID);
+
     QString info;
-    addIfNotEmpty(i18n("Reason:"), updateText, info);
-    addIfNotEmpty(i18n("Obsoletes:"), joinPackages(obsoletes), info);
-    addIfNotEmpty(i18n("Updates:"), joinPackages(updates), info);
+    addIfNotEmpty(i18n("Current Version:"), joinPackages(updates, {}, name), info);
+    addIfNotEmpty(i18n("Obsoletes:"), joinPackages(obsoletes, {}, name), info);
+    addIfNotEmpty(i18n("New Version:"), updateText, info);
     addIfNotEmpty(i18n("Update State:"), PackageKitMessages::updateStateMessage(state), info);
     addIfNotEmpty(i18n("Restart:"), PackageKitMessages::restartMessage(restart), info);
 
     if (!vendorUrls.isEmpty())
         addIfNotEmpty(i18n("Vendor:"), urlToLinks(vendorUrls).join(QStringLiteral(", ")), info);
 
-    emit changelogFetched(info);
+    emit changelogFetched(changelog() + info);
 }
 
 PackageKitBackend* PackageKitResource::backend() const
@@ -267,15 +284,21 @@ QString PackageKitResource::sizeDescription()
         return i18np("%2 (plus %1 dependency)", "%2 (plus %1 dependencies)", m_dependenciesCount, AbstractResource::sizeDescription());
 }
 
+QString PackageKitResource::sourceIcon() const
+{
+    return QStringLiteral("package-available");
+}
+
 void PackageKitResource::fetchDependencies()
 {
-    const auto id = availablePackageId();
+    const auto id = isInstalled() ? installedPackageId() : availablePackageId();
     if (id.isEmpty())
         return;
     m_dependenciesCount = 0;
 
     QSharedPointer<QJsonObject> packageDependencies(new QJsonObject);
-    auto trans = PackageKit::Daemon::installPackage(id, PackageKit::Transaction::TransactionFlagSimulate);
+
+    auto trans = PackageKit::Daemon::dependsOn(id);
     connect(trans, &PackageKit::Transaction::errorCode, this, [this](PackageKit::Transaction::Error, const QString& message) { qWarning() << "Transaction error: " << message << sender(); });
     connect(trans, &PackageKit::Transaction::package, this, [packageDependencies](PackageKit::Transaction::Info /*info*/, const QString &packageID, const QString &summary) {
         (*packageDependencies)[PackageKit::Daemon::packageName(packageID)] = summary ;
