@@ -30,6 +30,7 @@ class PackageKitBackend;
 class PackageKitResource : public AbstractResource
 {
     Q_OBJECT
+    Q_PROPERTY(QStringList objects MEMBER m_objects CONSTANT)
     public:
         explicit PackageKitResource(QString  packageName, QString  summary, PackageKitBackend* parent);
         QString packageName() const override;
@@ -45,9 +46,10 @@ class PackageKitResource : public AbstractResource
         bool isTechnical() const override;
         int size() override;
         void fetchChangelog() override;
+        void fetchUpdateDetails() override;
         
         QList<PackageState> addonsInformation() override;
-        State state() const override;
+        State state() override;
         
         QString installedVersion() const override;
         QString availableVersion() const override;
@@ -61,7 +63,7 @@ class PackageKitResource : public AbstractResource
 
         PackageKitBackend* backend() const;
 
-        static QString joinPackages(const QStringList& pkgids, const QString &_sep = {});
+        static QString joinPackages(const QStringList& pkgids, const QString &_sep, const QString &shadowPackageName);
 
         void invokeApplication() const override {}
         bool canExecute() const override { return false; }
@@ -69,6 +71,12 @@ class PackageKitResource : public AbstractResource
         QString sizeDescription() override;
         void setDependenciesCount(int count);
 
+        QString sourceIcon() const override;
+
+        QDate releaseDate() const override { return {}; }
+
+        virtual QString changelog() const { return {}; }
+
     Q_SIGNALS:
         void dependenciesFound(const QJsonObject& dependencies);
 
@@ -94,7 +102,6 @@ class PackageKitResource : public AbstractResource
 
     private:
         void fetchDependencies();
-        void setDependencies(const QStringList &deps);
         /** fetches details individually, it's better if done in batch, like for updates */
         void fetchDetails();
 
@@ -103,6 +110,7 @@ class PackageKitResource : public AbstractResource
         const QString m_name;
         PackageKit::Details m_details;
         int m_dependenciesCount = -1;
+        static const QStringList m_objects;
 };
 
 #endif // PACKAGEKITRESOURCE_H
