@@ -30,7 +30,7 @@
 #include "Transaction/TransactionModel.h"
 #include "Category/CategoryModel.h"
 #include "utils.h"
-#include <QDebug>
+#include "libdiscover_debug.h"
 #include <QCoreApplication>
 #include <QThread>
 #include <QAction>
@@ -72,7 +72,12 @@ void ResourcesModel::init(bool load)
     m_updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
     m_updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
     m_updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(this, &ResourcesModel::fetchingChanged, m_updateAction, [this](bool fetching) {
+        m_updateAction->setEnabled(!fetching);
+    });
     connect(m_updateAction, &QAction::triggered, this, &ResourcesModel::checkForUpdates);
+
+    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit, this, &QObject::deleteLater);
 }
 
 ResourcesModel::ResourcesModel(const QString& backendName, QObject* parent)
@@ -84,6 +89,7 @@ ResourcesModel::ResourcesModel(const QString& backendName, QObject* parent)
 
 ResourcesModel::~ResourcesModel()
 {
+    s_self = nullptr;
     qDeleteAll(m_backends);
 }
 
@@ -91,7 +97,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
 {
     Q_ASSERT(!m_backends.contains(backend));
     if(!backend->isValid()) {
-        qWarning() << "Discarding invalid backend" << backend->name();
+        qCWarning(LIBDISCOVER_LOG) << "Discarding invalid backend" << backend->name();
         CategoryModel::global()->blacklistPlugin(backend->name());
         backend->deleteLater();
         return;
@@ -124,7 +130,7 @@ void ResourcesModel::callerFetchingChanged()
     AbstractResourcesBackend* backend = qobject_cast<AbstractResourcesBackend*>(sender());
 
     if (!backend->isValid()) {
-        qWarning() << "Discarding invalid backend" << backend->name();
+        qCWarning(LIBDISCOVER_LOG) << "Discarding invalid backend" << backend->name();
         int idx = m_backends.indexOf(backend);
         Q_ASSERT(idx>=0);
         m_backends.removeAt(idx);
@@ -201,7 +207,7 @@ void ResourcesModel::registerAllBackends()
     DiscoverBackendsFactory f;
     const auto backends = f.allBackends();
     if(m_initializingBackends==0 && backends.isEmpty()) {
-        qWarning() << "Couldn't find any backends";
+        qCWarning(LIBDISCOVER_LOG) << "Couldn't find any backends";
         emit allInitialized();
     } else {
         foreach(AbstractResourcesBackend* b, backends) {
@@ -214,7 +220,8 @@ void ResourcesModel::registerAllBackends()
 void ResourcesModel::registerBackendByName(const QString& name)
 {
     DiscoverBackendsFactory f;
-    for(auto b : f.backend(name))
+    const auto backends = f.backend(name);
+    for(auto b : backends)
         addResourcesBackend(b);
 
     emit backendsChanged();
@@ -265,7 +272,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
 {
     Q_ASSERT(!streams.contains(nullptr));
     if (streams.isEmpty()) {
-        qWarning() << "no streams to aggregate!!";
+        qCWarning(LIBDISCOVER_LOG) << "no streams to aggregate!!";
         QTimer::singleShot(0, this, &AggregatedResultsStream::clear);
     }
 
@@ -279,8 +286,15 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     connect(&m_delayedEmission, &QTimer::timeout, this, &AggregatedResultsStream::emitResults);
 }
 
+AggregatedResultsStream::~AggregatedResultsStream() = default;
+
 void AggregatedResultsStream::addResults(const QVector<AbstractResource *>& res)
 {
+    for(auto r : res)
+        connect(r, &QObject::destroyed, this, [this, r](){
+            m_results.removeAll(r);
+        });
+
     m_results += res;
 
     m_delayedEmission.start();
@@ -335,8 +349,7 @@ AbstractResource* ResourcesModel::resourceForFile(const QUrl& file)
 void ResourcesModel::checkForUpdates()
 {
     for(auto backend: qAsConst(m_backends))
-        if (!backend->isFetching())
-            backend->checkForUpdates();
+        backend->checkForUpdates();
 }
 
 QVector<AbstractResourcesBackend *> ResourcesModel::applicationBackends() const
@@ -365,7 +378,7 @@ void ResourcesModel::setCurrentApplicationBackend(AbstractResourcesBackend* back
                 settings.deleteEntry("currentApplicationBackend");
         }
 
-        qDebug() << "setting currentApplicationBackend" << backend;
+        qCDebug(LIBDISCOVER_LOG) << "setting currentApplicationBackend" << backend;
         m_currentApplicationBackend = backend;
         Q_EMIT currentApplicationBackendChanged(backend);
     }
@@ -380,7 +393,7 @@ void ResourcesModel::initApplicationsBackend()
     auto idx = kIndexOf(backends, [name](AbstractResourcesBackend* b) { return b->name() == name; });
     if (idx<0) {
         idx = kIndexOf(backends, [](AbstractResourcesBackend* b) { return b->hasApplications(); });
-        qDebug() << "falling back applications backend to" << idx;
+        qCDebug(LIBDISCOVER_LOG) << "falling back applications backend to" << idx;
     }
     setCurrentApplicationBackend(backends.value(idx, nullptr), false);
 }
