@@ -26,7 +26,7 @@
 #include "AbstractResource.h"
 #include "utils.h"
 #include <QDateTime>
-#include <QDebug>
+#include "libdiscover_debug.h"
 
 #include <KLocalizedString>
 #include <KFormat>
@@ -42,8 +42,10 @@ public:
         bool cancelable = false;
         foreach(auto updater, m_allUpdaters) {
             connect(updater, &AbstractBackendUpdater::progressingChanged, this, &UpdateTransaction::slotProgressingChanged);
+            connect(updater, &AbstractBackendUpdater::downloadSpeedChanged, this, &UpdateTransaction::slotDownloadSpeedChanged);
             connect(updater, &AbstractBackendUpdater::progressChanged, this, &UpdateTransaction::slotUpdateProgress);
             connect(updater, &AbstractBackendUpdater::proceedRequest, this, &UpdateTransaction::processProceedRequest);
+            connect(updater, &AbstractBackendUpdater::cancelableChanged, this, [this](bool cancelable){ if (cancelable) setCancellable(true); });
             cancelable |= updater->isCancelable();
         }
         setCancellable(cancelable);
@@ -57,7 +59,7 @@ public:
     void cancel() override {
         QVector<AbstractBackendUpdater*> toCancel = m_updatersWaitingForFeedback.isEmpty() ? m_allUpdaters : m_updatersWaitingForFeedback;
 
-        foreach(auto updater, m_updatersWaitingForFeedback) {
+        foreach(auto updater, toCancel) {
             updater->cancel();
         }
     }
@@ -77,7 +79,7 @@ public:
 
     void slotProgressingChanged()
     {
-        if (status() < DoneStatus && !isProgressing()) {
+        if (status() > SetupStatus && status() < DoneStatus && !isProgressing()) {
             setStatus(Transaction::DoneStatus);
             Q_EMIT finished();
             deleteLater();
@@ -93,6 +95,15 @@ public:
         setProgress(total / m_allUpdaters.count());
     }
 
+    void slotDownloadSpeedChanged()
+    {
+        quint64 total = 0;
+        foreach(AbstractBackendUpdater* updater, m_allUpdaters) {
+            total += updater->downloadSpeed();
+        }
+        setDownloadSpeed(total);
+    }
+
     QVariant icon() const override { return QStringLiteral("update-low"); }
     QString name() const override { return i18n("Update"); }
 
@@ -158,7 +169,7 @@ void ResourcesUpdatesModel::message(const QString& msg)
 void ResourcesUpdatesModel::prepare()
 {
     if(isProgressing()) {
-        qWarning() << "trying to set up a running instance";
+        qCWarning(LIBDISCOVER_LOG) << "trying to set up a running instance";
         return;
     }
     foreach(AbstractBackendUpdater* upd, m_updaters) {
@@ -172,13 +183,23 @@ void ResourcesUpdatesModel::updateAll()
         delete m_transaction;
 
         const auto updaters = kFilter<QVector<AbstractBackendUpdater*>>(m_updaters, [](AbstractBackendUpdater* u) {return u->hasUpdates(); });
+        if (updaters.isEmpty()) {
+            return;
+        }
 
         m_transaction = new UpdateTransaction(this, updaters);
+        m_transaction->setStatus(Transaction::SetupStatus);
         setTransaction(m_transaction);
         TransactionModel::global()->addTransaction(m_transaction);
         Q_FOREACH (AbstractBackendUpdater* upd, updaters) {
             QMetaObject::invokeMethod(upd, "start", Qt::QueuedConnection);
         }
+
+        QMetaObject::invokeMethod(this, [this](){
+            m_transaction->setStatus(Transaction::CommittingStatus);
+            m_transaction->slotProgressingChanged();
+        }, Qt::QueuedConnection);
+
     }
 }
 
