@@ -20,9 +20,11 @@
 
 #include "DiscoverNotifier.h"
 #include "BackendNotifierFactory.h"
+#include <QDebug>
+#include <QDBusInterface>
+#include <QDBusPendingReply>
 #include <KConfig>
 #include <KConfigGroup>
-#include <QDebug>
 #include <KRun>
 #include <KLocalizedString>
 #include <KNotification>
@@ -37,10 +39,17 @@ DiscoverNotifier::DiscoverNotifier(QObject * parent)
     m_backends = BackendNotifierFactory().allBackends();
     foreach(BackendNotifierModule* module, m_backends) {
         connect(module, &BackendNotifierModule::foundUpdates, this, &DiscoverNotifier::updateStatusNotifier);
+        connect(module, &BackendNotifierModule::needsRebootChanged, this, [this]() {
+            if (!m_needsReboot) {
+                m_needsReboot = true;
+                showRebootNotification();
+                Q_EMIT updatesChanged();
+            }
+        });
     }
     connect(&m_timer, &QTimer::timeout, this, &DiscoverNotifier::showUpdatesNotification);
     m_timer.setSingleShot(true);
-    m_timer.setInterval(60000);
+    m_timer.setInterval(1000);
     updateStatusNotifier();
 }
 
@@ -54,7 +63,12 @@ void DiscoverNotifier::configurationChanged()
     m_verbose = notifyTypeGroup.readEntry("Verbose", false);
 }
 
-void DiscoverNotifier::showMuon()
+void DiscoverNotifier::showDiscover()
+{
+    KRun::runCommand(QStringLiteral("plasma-discover"), nullptr);
+}
+
+void DiscoverNotifier::showDiscoverUpdates()
 {
     KRun::runCommand(QStringLiteral("plasma-discover --mode update"), nullptr);
 }
@@ -70,7 +84,7 @@ void DiscoverNotifier::showUpdatesNotification()
     const QString name = i18n("Update");
     e->setDefaultAction(name);
     e->setActions({name});
-    connect(e, QOverload<unsigned int>::of(&KNotification::activated), this, &DiscoverNotifier::showMuon);
+    connect(e, QOverload<unsigned int>::of(&KNotification::activated), this, &DiscoverNotifier::showDiscoverUpdates);
 }
 
 void DiscoverNotifier::updateStatusNotifier()
@@ -97,7 +111,9 @@ void DiscoverNotifier::updateStatusNotifier()
 
 DiscoverNotifier::State DiscoverNotifier::state() const
 {
-    if (m_securityCount)
+    if (m_needsReboot)
+        return RebootRequired;
+    else if (m_securityCount)
         return SecurityUpdates;
     else if (m_count)
         return NormalUpdates;
@@ -114,6 +130,8 @@ QString DiscoverNotifier::iconName() const
             return QStringLiteral("update-low");
         case NoUpdates:
             return QStringLiteral("update-none");
+        case RebootRequired:
+            return QStringLiteral("system-reboot");
     }
     return QString();
 }
@@ -127,6 +145,8 @@ QString DiscoverNotifier::message() const
             return i18n("Updates available");
         case NoUpdates:
             return i18n("System up to date");
+        case RebootRequired:
+            return i18n("Computer needs to restart");
     }
     return QString();
 }
@@ -174,3 +194,24 @@ QStringList DiscoverNotifier::loadedModules() const
         ret += QString::fromLatin1(module->metaObject()->className());
     return ret;
 }
+
+void DiscoverNotifier::showRebootNotification()
+{
+    KNotification *notification = new KNotification(QLatin1String("notification"), KNotification::Persistent | KNotification::DefaultEvent);
+    notification->setIconName(QStringLiteral("system-software-update"));
+    notification->setActions(QStringList{QLatin1String("Restart")});
+    notification->setTitle(i18n("Restart is required"));
+    notification->setText(i18n("The system needs to be restarted for the updates to take effect."));
+
+    connect(notification, &KNotification::action1Activated, this, [] () {
+        QDBusInterface interface(QStringLiteral("org.kde.ksmserver"), QStringLiteral("/KSMServer"), QStringLiteral("org.kde.KSMServerInterface"), QDBusConnection::sessionBus());
+        interface.asyncCall(QStringLiteral("logout"), 0, 1, 2); // Options: do not ask again | reboot | force
+    });
+
+    notification->sendEvent();
+}
+
+bool DiscoverNotifier::isSystemUpToDate() const
+{
+    return m_count==0 && m_securityCount==0;
+}
